import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {Constants} from "src/utils/constants";
import {HomePageHeroSectionCmsService} from "./home-page-hero-section-cms.service";
import {UpdateHomePageHeroSectionCmsDto} from "./dto/update-home-page-hero-section-cms.dto";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Controller({path: "home-page-hero-section-cms", version: Constants.API_VERSION_1})

export class HomePageHeroSectionCmsController {
    constructor(private readonly service: HomePageHeroSectionCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "overviewImage", maxCount: 1}
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateHomePageHeroSectionCmsDto,
        @UploadedFiles() files: {
            overviewImage: UploadedMulterFileI
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
