import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {SchedulePageCmsRepository} from "./schedule-page-cms.repository";
import {UpdateSchedulePageCmsDto} from "./dto/update-schedule-page-cms.dto";
import {SchedulePageCms} from "./entities/schedule-page-cms.entity";

@Injectable()
export class SchedulePageCmsService {
    constructor(
        @InjectRepository(SchedulePageCmsRepository)
        private readonly repository: SchedulePageCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateSchedulePageCmsDto,
        files: {
            icon: UploadedMulterFileI
        }
    ): Promise<SchedulePageCms | Error> {
        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "SchedulePageCms/");
            dto.icon = icon;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<SchedulePageCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}