import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {TermsPageCmsRepository} from "./terms-page-cms.repository";
import {UpdateTermsPageCmsDto} from "./dto/update-terms-page-cms.dto";
import {TermsPageCmsEntity} from "./entities/terms-page-cms.entity";

@Injectable()
export class TermsPageCmsService {
    constructor(
        @InjectRepository(TermsPageCmsRepository)
        private readonly repository: TermsPageCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateTermsPageCmsDto,
        files: {
            termsofUseIcon: UploadedMulterFileI
            file: UploadedMulterFileI
        }
    ): Promise<TermsPageCmsEntity | Error> {
        if (files && files.termsofUseIcon) {
            const termsofUseIcon: any = await this.doSpaceService.uploadFile(files.termsofUseIcon[0], "TermsPageCms/");
            dto.termsofUseIcon = termsofUseIcon;
        }

        if (files && files.file) {
            const file: any = await this.doSpaceService.uploadFile(files.file[0], "TermsPageCms/");
            dto.file = file;
        }

        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<TermsPageCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}