import {
    Body,
    Controller,
    Get,
    HttpCode,
    HttpStatus,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from "@nestjs/common";
import {AdminAuthGuard} from "src/core/guards/admin-auth.guard";
import {Constants} from "src/utils/constants";
import {WhatsNextSectionCmsService} from "./whats-next-section-cms.service";
import {UpdateWhatsNextSectionCmsDto} from "./dto/update-whats-next-section-cms.dto";
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";

@Controller({path: "whats-next-section-cms", version: Constants.API_VERSION_1})

export class WhatsNextSectionCmsController {
    constructor(private readonly service: WhatsNextSectionCmsService) {
    }

    @Post()
    @HttpCode(HttpStatus.OK)
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            {name: "sectionOneImage", maxCount: 1},
            {name: "sectionTwoImage", maxCount: 1},
        ]
    ))
    updateOrCreateData(
        @Body() dto: UpdateWhatsNextSectionCmsDto,
        @UploadedFiles() files: {
            sectionOneImage: UploadedMulterFileI,
            sectionTwoImage: UploadedMulterFileI,
        }
    ) {
        return this.service.updateOrCreateData(dto, files);
    }

    @Get()
    // @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }
}
