import {HttpException, HttpStatus, Injectable} from "@nestjs/common";
import {InjectRepository} from "@nestjs/typeorm";
import {Constants} from "src/utils/constants";
import {ResponseUtils} from "src/utils/response.utils";
import {WhatsNextSectionCmsRepository} from "./whats-next-section-cms.repository";
import {UpdateWhatsNextSectionCmsDto} from "./dto/update-whats-next-section-cms.dto";
import {WhatsNextSectionCmsEntity} from "./entities/whats-next-section-cms.entity";
import {UploadedMulterFileI} from "src/modules/SpacesModule/SpacesService";
import {DoSpacesService} from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class WhatsNextSectionCmsService {
    constructor(
        @InjectRepository(WhatsNextSectionCmsRepository)
        private readonly repository: WhatsNextSectionCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateWhatsNextSectionCmsDto,
        files: {
            sectionOneImage: UploadedMulterFileI,
            sectionTwoImage: UploadedMulterFileI,
        }
    ): Promise<WhatsNextSectionCmsEntity | Error> {
        if (files && files.sectionOneImage) {
            const sectionOneImage: any = await this.doSpaceService.uploadFile(files.sectionOneImage[0], "WhatsNextSectionCms/");
            dto.sectionOneImage = sectionOneImage;
        }
        if (files && files.sectionTwoImage) {
            const sectionTwoImage: any = await this.doSpaceService.uploadFile(files.sectionTwoImage[0], "WhatsNextSectionCms/");
            dto.sectionTwoImage = sectionTwoImage;
        }
        const isExists = await this.repository.find({isDeleted: false});
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<WhatsNextSectionCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}