import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateAboutPageContentDto} from "./dto/create-about-page-content.dto";
import {UpdateAboutPageContentDto} from "./dto/update-about-page-content.dto";
import {AboutPageContentEntity} from "./entity/about-page-content.entity";

@EntityRepository(AboutPageContentEntity)
export class AboutPageContentRepository extends Repository<AboutPageContentEntity> {

    async createEntity(dto: CreateAboutPageContentDto): Promise<AboutPageContentEntity | Error> {
        try {
            const {
                year,
                description
            } = dto;
            const entity = this.create({
                year,
                description
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<AboutPageContentEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<AboutPageContentEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateAboutPageContentDto): Promise<AboutPageContentEntity | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(AboutPageContentEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}