import {
    Body,
    Controller,
    Delete,
    Get,
    Param,
    Patch,
    Post,
    UploadedFiles,
    UseGuards,
    UseInterceptors
} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {BenefitsAndAdvantageSectionService} from './benefits-and-advantage-section.service';
import {CreateBenefitsAndAdvantageSectionDto} from './dto/create-benefits-and-advantage-section.dto';
import {UpdateBenefitsAndAdvantageSectionDto} from './dto/update-benefits-and-advantage-section.dto';
import {FileFieldsInterceptor} from "@nestjs/platform-express";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Controller({path: 'benefits-and-advantage-section', version: Constants.API_VERSION_1})
export class BenefitsAndAdvantageSectionController {
    constructor(private readonly service: BenefitsAndAdvantageSectionService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'image', maxCount: 1}
    ]))
    create(
        @Body() dto: CreateBenefitsAndAdvantageSectionDto,
        @UploadedFiles() files: {
            image?: UploadedMulterFileI,
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor([
        {name: 'image', maxCount: 1}
    ]))
    update(
        @Param('id') id: string,
        @Body() dto: UpdateBenefitsAndAdvantageSectionDto,
        @UploadedFiles() files: {
            image?: UploadedMulterFileI,
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
