import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateCareerApplicationModelDto} from "./dto/create-career-application-model.dto";
import {UpdateCareerApplicationModelDto} from "./dto/update-career-application-model.dto";
import {CareerApplicationModelEntity} from "./entity/career-application-model.entity";

@EntityRepository(CareerApplicationModelEntity)
export class CareerApplicationModelRepository extends Repository<CareerApplicationModelEntity> {
    async createEntity(dto: CreateCareerApplicationModelDto): Promise<CareerApplicationModelEntity | Error> {
        try {
            const {
                jobTitle,
                name,
                email,
                coverLetter,
                resume,
                reason
            } = dto;
            const entity = this.create({
                jobTitle,
                name,
                email,
                coverLetter,
                resume,
                reason
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<CareerApplicationModelEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateCareerApplicationModelDto): Promise<CareerApplicationModelEntity | Error> {
        try {
            await this.update(id, {
                jobTitle: dto.jobTitle,
                name: dto.name,
                email: dto.email,
                coverLetter: dto.coverLetter,
                resume: dto.resume,
                reason: dto.reason
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<CareerApplicationModelEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(CareerApplicationModelEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async countAllQuery(): Promise<any | Error> {
        try {
            return await this.count({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}