import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {CustomerReviewsRepository} from './customer-reviews.repository';
import {CreateCustomerReviewsDto} from './dto/create-customer-reviews.dto';
import {UpdateCustomerReviewsDto} from './dto/update-customer-reviews.dto';
import {CustomerReviewsEntity} from './entity/customer-reviews.entity';
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Injectable()
export class CustomerReviewsService {
    constructor(
        @InjectRepository(CustomerReviewsRepository)
        private readonly repository: CustomerReviewsRepository,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(dto: CreateCustomerReviewsDto, files: {
        userPhoto?: UploadedMulterFileI,
        icon?: UploadedMulterFileI,
    }): Promise<CustomerReviewsEntity | Error> {
        if (files && files.userPhoto) {
            const url = await this.doSpacesService.uploadFile(files.userPhoto[0], "CustomerReviews/");
            dto.userPhoto = url.toString();
        }
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "CustomerReviews/");
            dto.icon = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(): Promise<CustomerReviewsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateCustomerReviewsDto, files: {
        userPhoto?: UploadedMulterFileI,
        icon?: UploadedMulterFileI,
    }): Promise<CustomerReviewsEntity | Error> {
        if (files && files.userPhoto) {
            const url = await this.doSpacesService.uploadFile(files.userPhoto[0], "CustomerReviews/");
            dto.userPhoto = url.toString();
        }
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "CustomerReviews/");
            dto.icon = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<CustomerReviewsEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
