import {EntityRepository, getConnection, Repository} from "typeorm";
import {CreateHowItWorksSectionDto} from "./dto/create-how-it-works-section.dto";
import {UpdateHowItWorksSectionDto} from "./dto/update-how-it-works-section.dto";
import {HowItWorksSectionEntity} from "./entity/how-it-works-section.entity";

@EntityRepository(HowItWorksSectionEntity)
export class HowItWorksSectionRepository extends Repository<HowItWorksSectionEntity> {

    async createEntity(dto: CreateHowItWorksSectionDto): Promise<HowItWorksSectionEntity | Error> {
        try {
            const {
                icon,
                title,
                description,
            } = dto;
            const entity = this.create({
                icon,
                title,
                description,
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<HowItWorksSectionEntity[] | Error> {
        try {
            return await this.find({
                where: {...query, isDeleted: false}
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<HowItWorksSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateHowItWorksSectionDto): Promise<HowItWorksSectionEntity | Error> {
        try {
            await this.update(id, {
                icon: dto.icon,
                title: dto.title,
                description: dto.description,
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(HowItWorksSectionEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}