import {HttpException, HttpStatus, Injectable} from '@nestjs/common';
import {InjectRepository} from '@nestjs/typeorm';
import {Constants} from 'src/utils/constants';
import {ResponseUtils} from 'src/utils/response.utils';
import {HowItWorksSectionRepository} from './how-it-works-section.repository';
import {CreateHowItWorksSectionDto} from './dto/create-how-it-works-section.dto';
import {UpdateHowItWorksSectionDto} from './dto/update-how-it-works-section.dto';
import {HowItWorksSectionEntity} from './entity/how-it-works-section.entity';
import {DoSpacesService} from "../../SpacesModule/SpacesService/doSpacesService";
import {UploadedMulterFileI} from "../../SpacesModule/SpacesService";

@Injectable()
export class HowItWorksSectionService {
    constructor(
        @InjectRepository(HowItWorksSectionRepository)
        private readonly repository: HowItWorksSectionRepository,
        private readonly doSpacesService: DoSpacesService
    ) {
    }

    async create(dto: CreateHowItWorksSectionDto, files: {
        icon?: UploadedMulterFileI,
    }): Promise<HowItWorksSectionEntity | Error> {
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "HowItWorksSection/");
            dto.icon = url.toString();
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data Created Successfully", "data", data);
    }

    async findAll(): Promise<HowItWorksSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async update(id: string, dto: UpdateHowItWorksSectionDto, files: {
        icon?: UploadedMulterFileI,
    }): Promise<HowItWorksSectionEntity | Error> {
        if (files && files.icon) {
            const url = await this.doSpacesService.uploadFile(files.icon[0], "BenefitsAndAdvantageSection/");
            dto.icon = url.toString();
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated Successfully", "data", data);
    }

    async findOne(id: string): Promise<HowItWorksSectionEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({id: id});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Fetched Successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data Deleted Successfully");
    }
}
