import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {ScheduleIndustryService} from './schedule-industry.service';
import {CreateScheduleIndustryDto} from './dto/create-schedule-industry.dto';
import {UpdateScheduleIndustryDto} from './dto/update-schedule-industry.dto';

@Controller({path: 'schedule-industry', version: Constants.API_VERSION_1})
export class ScheduleIndustryController {
    constructor(private readonly service: ScheduleIndustryService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateScheduleIndustryDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateScheduleIndustryDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
