import {Body, Controller, Delete, Get, Param, Patch, Post, UseGuards} from '@nestjs/common';
import {AdminAuthGuard} from 'src/core/guards/admin-auth.guard';
import {Constants} from 'src/utils/constants';
import {ScheduleTransactionLimitService} from './schedule-transaction-limit.service';
import {CreateScheduleTransactionLimitDto} from './dto/create-schedule-transaction-limit.dto';
import {UpdateScheduleTransactionLimitDto} from './dto/update-schedule-transaction-limit.dto';

@Controller({path: 'schedule-transaction-limit', version: Constants.API_VERSION_1})
export class ScheduleTransactionLimitController {
    constructor(private readonly service: ScheduleTransactionLimitService) {
    }

    @Post()
    @UseGuards(AdminAuthGuard)
    create(@Body() dto: CreateScheduleTransactionLimitDto) {
        return this.service.create(dto);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(':id')
    @UseGuards(AdminAuthGuard)
    findOne(@Param('id') id: string) {
        return this.service.findOne(id);
    }

    @Patch(':id')
    @UseGuards(AdminAuthGuard)
    update(@Param('id') id: string, @Body() dto: UpdateScheduleTransactionLimitDto) {
        return this.service.update(id, dto);
    }

    @Delete(':id')
    @UseGuards(AdminAuthGuard)
    async remove(@Param('id') id: string) {
        return await this.service.remove(id);
    }
}
