import { EntityRepository, getConnection, Repository, Like, Between, getManager } from "typeorm";
import { LogsEntity } from "./entity/logs.entity";
import { CreateLogDto } from "./dto/create-log.dto";

@EntityRepository(LogsEntity)
export class LogsRepository extends Repository<LogsEntity> {

    async createEntity(dto: CreateLogDto): Promise<LogsEntity | Error> {
        try {
            const {
                userId,
                address,
                clientName,
                informationChnage,
                reasonForChanging,
                status
            } = dto;
            const entity = this.create({
                userId,
                address,
                clientName,
                informationChnage,
                reasonForChanging,
                status
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    // async findByFilterQuery(query: any, offset: number, limit: number): Promise<LogsEntity[] | Error> {
    //     try {
    //         return await this.find({
    //             where: { ...query, isDeleted: false },
    //             order: { createdAt: -1 },
    //             skip: offset,
    //             take: limit,
    //         });
    //     } catch (e) {
    //         return e;
    //     }
    // }


    // async findByFilterQuery(query: any, offset: number, limit: number, searchName?: string | Date): Promise<LogsEntity[] | Error> {
    //     try {
    //         const filterQuery: any = { ...query, isDeleted: false };

    //         if (searchName) {
    //             const clientNameColumn = "clientName";

    //             if (searchName instanceof Date) {
    //                 // For Date type, we can use the 'Between' operator to search within a range
    //                 return await this.find({
    //                     where: {
    //                         ...filterQuery,
    //                         userId: query.userId, // Filter based on the userId
    //                         [clientNameColumn]: Between(searchName.toISOString(), new Date(new Date(searchName).setDate(searchName.getDate() + 1)).toISOString()),
    //                     },
    //                     order: { createdAt: -1 },
    //                     skip: offset,
    //                     take: limit,
    //                 });
    //             } else if (typeof searchName === 'string') {
    //                 // For string type, we search for a case-insensitive match
    //                 return await this.find({
    //                     where: {
    //                         ...filterQuery,
    //                         userId: query.userId, // Filter based on the userId
    //                         [clientNameColumn]: Like(`%${searchName}%`),
    //                     },
    //                     order: { createdAt: -1 },
    //                     skip: offset,
    //                     take: limit,
    //                 });
    //             }
    //         }

    //         // If searchName is not provided, only filter by userId
    //         return await this.find({
    //             where: { ...filterQuery, userId: query.userId },
    //             order: { createdAt: -1 },
    //             skip: offset,
    //             take: limit,
    //         });
    //     } catch (e) {
    //         return e;
    //     }
    // }



    // async findByFilterQuery(query: any, offset: number, limit: number, searchName ?: string | Date): Promise < LogsEntity[] | Error > {
    //     try {
    //         const filterQuery: any = { ...query, isDeleted: false };

    //         if(searchName) {
    //             const columnsToSearch = ["clientName", "address", "informationChnage", "createdAt"]; // Add more columns if needed

    //             // Iterate through columnsToSearch and apply filtering to the first matched column
    //             for (const column of columnsToSearch) {
    //                 if (filterQuery[column] instanceof Date) {
    //                     console.log('if', searchName)
    //                     const searchDate = new Date(searchName); // Convert searchName to Date
    //                     const nextDay = new Date(searchDate);
    //                     nextDay.setDate(nextDay.getDate() + 1);

    //                     // For Date type, use the 'Between' operator to search within a range
    //                     filterQuery[column] = Between(searchDate.toISOString(), nextDay.toISOString());
    //                     filterQuery.userId = query.userId; // Filter based on the userId
    //                     break; // Apply filtering to the first matched column
    //                 } else if (typeof filterQuery[column] === 'string') {
    //                     console.log('elseif', searchName)
    //                     // For string type, search for a case-insensitive match
    //                     filterQuery[column] = Like(`%${searchName}%`);
    //                     filterQuery.userId = query.userId; // Filter based on the userId
    //                     break; // Apply filtering to the first matched column
    //                 }
    //             }
    //         } else {
    //             console.log('else', searchName)
    //             // If searchName is not provided, only filter by userId
    //             filterQuery.userId = query.userId;
    //         }

    //         console.log('outsite', searchName)

    //         return await this.find({
    //             where: filterQuery,
    //             order: { createdAt: -1 },
    //             skip: offset,
    //             take: limit,
    //         });
    //     } catch(e) {
    //         return e;
    //     }
    // }


    async findByFilterQuery(userId: string, paginationDto: any): Promise<any | Error> {
        try {
            const { page, limit, searchName, startDate, endDate, sortName, sort } = paginationDto;
            let whereQuery: any = `l.id in (SELECT id  FROM logs WHERE userId='${userId}')`;
            let orderByQuery = `ORDER BY createdAt DESC`;

            if (sortName != "Default" && sortName != "" && sort != "") {
                orderByQuery = `ORDER BY ${sortName} ${sort}`
            }

            if (searchName) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and l.clientName LIKE '%${searchName}%' OR
                    l.address LIKE '%${searchName}%' OR 
                    l.informationChnage LIKE '%${searchName}%' OR 
                    l.reasonForChanging LIKE '%${searchName}%' OR
                    l.status LIKE '%${searchName}%' `;
                } else {
                    whereQuery += ` l.clientName LIKE '%${searchName}%' OR 
                    l.address LIKE '%${searchName}%' OR
                    l.informationChnage LIKE '%${searchName}%' OR
                    l.reasonForChanging LIKE '%${searchName}%' OR
                    l.status LIKE '%${searchName}%' `;
                }
            }

            if (startDate && endDate) {
                if (whereQuery.length > 3) {
                    whereQuery += ` and l.createdAt between '${startDate}' and '${endDate}'`;
                } else {
                    whereQuery += ` l.createdAt between '${startDate}' and '${endDate}'`;
                }
            }

            if (whereQuery.length > 3) {
                whereQuery += ` and l.isDeleted=0 `;
            } else {
                whereQuery += ` l.isDeleted=0 `;
            }

            if (whereQuery.substring(0, 3) === 'and') {
                whereQuery = whereQuery.slice(4, whereQuery.length);
            }

            let conditionQuery = ``;

            if (!whereQuery) {
                conditionQuery = ``;
            } else {
                conditionQuery = ` WHERE ${whereQuery} `;
            }

            let offset = 0;

            if (page > 1) {
                offset = (page - 1) * limit;
            }

            let sql = `SELECT l.* FROM logs as l ${conditionQuery} ${orderByQuery} limit ${offset}, ${limit};`;
            let sqlTotal = `SELECT count(*) total FROM logs as l ${conditionQuery}`;

            // console.log(sql);
            // console.log(sqlTotal);
            const entityManager = getManager();
            const data = await entityManager.query(sql);
            const totalData = await entityManager.query(sqlTotal);
            const count = totalData?.[0]?.total;

            if (count > 0) {
                const pageCount = Math.ceil(count / limit);

                return {
                    page: page,
                    limit: limit,
                    pageCount: pageCount,
                    totalDataFound: count,
                    data,
                };
            } else {
                return {};
            }
        } catch (e) {
            console.log(e);
            return e;
        }
    }


    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(LogsEntity)
                .set({isDeleted: true})
                .where("id = :id", {id: id})
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}