import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Constants } from 'src/utils/constants';
import { ResponseUtils } from 'src/utils/response.utils';
import { TransactionRepository } from '../transaction/transaction.repository';
import { LogsRepository } from '../transaction-logs/logs.repository';
import { CreateTransactionPaymentHistoryDto } from './dto/create-transaction-payment-history.dto';
import { TransactionPaymentHistoryEntity } from './entity/transaction-payment-history.entity';
import { UpdateTransactionPaymentHistoryDto } from './dto/update-transaction-payment-history.dto';
import { TransactionPaymentHistoryRepository } from './transaction-payment-history.repository';

@Injectable()
export class TransactionPaymentHistoryService {
    constructor(
        @InjectRepository(TransactionPaymentHistoryRepository)
        private readonly repository: TransactionPaymentHistoryRepository,
        @InjectRepository(TransactionRepository)
        private readonly transactionRepository: TransactionRepository,
        @InjectRepository(LogsRepository)
        private readonly logsRepository: LogsRepository
    ) {
    }

    async create(
        dto: CreateTransactionPaymentHistoryDto,
        request: any
    ): Promise<TransactionPaymentHistoryEntity | Error> {
        const transaction: any = await this.transactionRepository.findByFilterQuery({ id: dto.transactionId });
        if (!transaction.length) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
    }

    async update(
        id: string,
        dto: UpdateTransactionPaymentHistoryDto,
        user: any
    ): Promise<TransactionPaymentHistoryEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        const { transactionId, ...updateDto } = dto;
        const data = await this.repository.updateEntity(id, updateDto);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        // find specifice transaction
        const tran = await this.transactionRepository.findOneByFilterQuery({ id: res.transactionId })
        // create logs
        const createLogsDto = {
            userId: tran[0].userId,
            address: tran[0].streetAddress,
            clientName: tran[0].clientName,
            informationChnage: "Updated transaction timeline information",
            reasonForChanging: null,
            status: null
        }
        await this.logsRepository.createEntity(createLogsDto);

        return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
    }

    async findAll(): Promise<TransactionPaymentHistoryEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<TransactionPaymentHistoryEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
