export class ResponseUtils {
    static buildData(data: any): any {
        return {
            data
        };
    }

    static buildDeletedData(
        data: any,
        statusCode?: Number,
        message?: String
    ): any {
        return {
            message: message,
            statusCode: statusCode,
            deleted: data
        };
    }

    static successResponseHandler(statusCode: number, message: string, keyOfData?: (string | null), data?: any): any {
        let responseResult: {
            status: String;
            statusCode: Number;
            message: String;
            [key: string]: any;
        } = {
            message: message,
            status: "success",
            statusCode: statusCode
        };

        if (keyOfData && data) responseResult[keyOfData] = data;
        if (!keyOfData && data) responseResult = {...responseResult, ...data};

        return responseResult;
    };

    static successResponseHandlerForQueryBuilder(
        statusCode: number,
        message: string,
        keyOfData?: string | null,
        data?: any,
        paginationData?: { page: number; limit: number; totalCount: number }
    ): any {
        let responseResult: {
            status: string;
            statusCode: number;
            message: string;
            [key: string]: any;
        } = {
            message: message,
            status: "success",
            statusCode: statusCode
        };

        if (keyOfData && data) responseResult[keyOfData] = data;
        if (!keyOfData && data) responseResult = { ...responseResult, ...data };

        if (paginationData) {
            responseResult.paginationData = paginationData;
        }

        return responseResult;
    }

    static errorResponseHandler(statusCode: number, message: string, keyOfData?: (string | null), data?: any): any {
        let responseResult: {
            status: String;
            statusCode: Number;
            message: String;
            [key: string]: any;
        } = {
            message: message,
            status: "failed",
            statusCode: statusCode
        };

        if (keyOfData && data) responseResult[keyOfData] = data;
        if (!keyOfData && data) responseResult = {...responseResult, ...data};

        return responseResult;
    };
}
